import logging
from io import TextIOWrapper
from typing import Optional

import click
import click_completion

from jenkinsctl.commands.build import build_handler
from jenkinsctl.commands.config import config_handler
from jenkinsctl.commands.enable_completion import handle_enable_completion
from jenkinsctl.commands.jobs import jobs_handler
from jenkinsctl.commands.json import json_handler
from jenkinsctl.commands.list import list_handler
from jenkinsctl.commands.logs import logs_handler
from jenkinsctl.commands.rebuild import rebuild_handler
from jenkinsctl.configs.logging_config import setup_logging
from jenkinsctl.jenkins.cli_helper import error_handler_and_session

click_completion.init()


@click.group()
@click.option('-v', '--verbose', is_flag=True, help="Enable verbose output")
@click.pass_context
def cli(ctx: click.Context, verbose: bool) -> None:
    """A command-line tool to interact with Jenkins jobs"""
    log_level: int = logging.DEBUG if verbose else logging.INFO
    ctx.logger = setup_logging(log_level)


@cli.command("list")
@click.argument("job_name")
@click.option("-n", "--number", default=5, type=int, help="No of builds to list (default: 5)")
def list_command(job_name: str, number: int) -> None:
    """
    List all builds of a Jenkins job

    \b
    JOB_NAME: Name of the Jenkins job
    """
    with error_handler_and_session() as session:
        list_handler(session, job_name, number)


@cli.command("logs")
@click.argument("job_name")
@click.argument("build_no", required=False, type=int)
def logs_command(job_name: str, build_no: Optional[int]) -> None:
    """
    Print logs of a Jenkins build

    \b
    JOB_NAME: Name of the Jenkins job
    BUILD_NO: Build number (default: last build)
    """
    with error_handler_and_session() as session:
        logs_handler(session, job_name, build_no)


@cli.command("json")
@click.argument("job_name")
@click.argument("build_no", required=False, type=int)
def json_command(job_name: str, build_no: Optional[int]) -> None:
    """
    Fetch and print the JSON API response of a Jenkins build

    \b
    JOB_NAME: Name of the Jenkins job
    BUILD_NO: Build number (default: last build)
    """
    with error_handler_and_session() as session:
        json_handler(session, job_name, build_no)


@cli.command("config")
@click.argument("job_name")
@click.argument("build_no", required=False, type=int)
def config_command(job_name: str, build_no: Optional[int]) -> None:
    """
    Get the configuration of a specific build in YAML format

    \b
    JOB_NAME: Name of the Jenkins job
    BUILD_NO: Build number (default: last build)
    """
    with error_handler_and_session() as session:
        config_handler(session, job_name, build_no)


@cli.command("rebuild")
@click.argument("job_name")
@click.argument("build_no", required=False, type=int)
def rebuild_command(job_name: str, build_no: Optional[int]) -> None:
    """
    Rebuild a specific Jenkins job

    \b
    JOB_NAME: Name of the Jenkins job
    BUILD_NO: Build number (default: last build)
    """
    with error_handler_and_session() as session:
        rebuild_handler(session, job_name, build_no)


@cli.command("build")
@click.option("-p", "--param", multiple=True,
              help="Override parameters in the YAML configuration (e.g., --param key=value)")
@click.option("-f", "--file", type=click.File('r'), required=True, help="YAML configuration file for the Jenkins job")
def build_command(file: TextIOWrapper, param: tuple[str]) -> None:
    """
    Trigger a new Jenkins build
    """
    params = [p for p in param]
    with error_handler_and_session() as session:
        build_handler(session, file, params)


@cli.command("enable-completion")
@click.argument('shell', required=False)
def enable_completion(shell: str| None):
    """
    Enable shell autocompletion for jenkinsctl.

    This command allows you to set up autocompletion for the jenkinsctl command-line tool,
    making it easier to use by providing suggestions as you type.

    \b
    SHELL: Specify a shell for which to enable autocompletion.
    Supported options include:
      - bash
      - zsh
      - fish

    If no shell is specified, it will attempt to autodetect your shell.
    """
    if shell is None:
        shell = click_completion.core.get_auto_shell()  # Detect the current shell

    handle_enable_completion(shell)


@cli.command("jobs")
@click.argument("folder_name", default="")
def jobs_command(folder_name: str) -> None:
    """
        List all jobs in a Jenkins folder.

        \b
        FOLDER_NAME: Path of the folder to list jobs (e.g., projectA/subProjectX).
                     If no folder is specified, it lists jobs in the root folder.
    """
    with error_handler_and_session() as session:
        print("hello")
        jobs_handler(session, folder_name)


# Entry point
if __name__ == '__main__':
    cli()



import logging
import sys

def setup_logging(log_level=logging.INFO):
    logger = logging.getLogger()
    logger.setLevel(log_level)

    console_handler = logging.StreamHandler(sys.stdout)
    console_handler.setLevel(log_level)
    console_handler.setFormatter(logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s"))

    file_handler = logging.FileHandler('jenkinsctl.log')
    file_handler.setLevel(log_level)
    file_handler.setFormatter(logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s"))

    logger.addHandler(console_handler)
    logger.addHandler(file_handler)

    return logger


def handle_enable_completion(shell: str):
    if shell == "bash":
        print("\n# To enable autocompletion for bash, add the following to your ~/.bashrc:")
        print("echo 'eval \"$(_JENKINSCTL_COMPLETE=bash_source jenkinsctl)\"' >> ~/.bashrc")
        print("\n# Then reload your bash configuration:")
        print("source ~/.bashrc")

    elif shell == "zsh":
        print("\n# To enable autocompletion for zsh, add the following to your ~/.zshrc:")
        print("echo 'eval \"$(_JENKINSCTL_COMPLETE=zsh_source jenkinsctl)\"' >> ~/.zshrc")
        print("\n# Then reload your zsh configuration:")
        print("source ~/.zshrc")

    elif shell == "fish":
        print(
            "\n# To enable autocompletion for fish, add the following to your Fish config (~/.config/fish/config.fish):")
        print("echo 'eval (jenkinsctl enable-completion fish | source)' >> ~/.config/fish/config.fish")
        print("\n# Then reload your fish shell configuration:")
        print("source ~/.config/fish/config.fish")

    else:
        print("\n# Unsupported shell. Please specify bash, zsh, or fish.")



import yaml


def get_build_params(build_json):
    actions = build_json["actions"]
    params = next((action["parameters"] for action in actions if action.get("parameters") is not None), [])
    params = dict([(param["name"], param["value"]) for param in params])
    return params


def normalize_job_path(path: str):
    return (path.strip()
            .removeprefix("/")
            .replace("/", "/job/"))


def print_build(build_json):
    number = build_json["number"]
    building = build_json["building"]
    in_progress = build_json["inProgress"]
    result = build_json["result"]
    timestamp = build_json["timestamp"]

    actions = build_json["actions"]
    causes = next((action["causes"] for action in actions if action.get("causes") is not None), None)
    user_id = next((cause["userId"] for cause in causes if cause.get("userId") is not None), None)

    print(number, building, in_progress, result, timestamp, user_id)
    print(get_build_params(build_json))


def get_last_build(job) -> int:
    return job["lastBuild"]["number"]


def to_yaml(job_name, params):
    return yaml.dump({"job": job_name, "params": params})



from jenkinsctl.jenkins.job import get_job
from jenkinsctl.jenkins.utils import get_last_build


def get_last_build_no_if_none(session, job_name, build_no):
    if build_no is None:
        job = get_job(session, job_name)
        return get_last_build(job)
    return build_no



from jenkinsctl.configs.session import Session
from jenkinsctl.jenkins.job import get_jobs
from jenkinsctl.jenkins.utils import normalize_job_path


def jobs_handler(session: Session, folder_name: str):
    folder_name = normalize_job_path(folder_name)
    response = get_jobs(session, folder_name)
    all_jobs = response["jobs"]

    folders = [job for job in all_jobs if job["_class"] == "com.cloudbees.hudson.plugins.folder.Folder"]
    jobs = [job for job in all_jobs if job["_class"] != "com.cloudbees.hudson.plugins.folder.Folder"]

    for folder in folders:
        folder_name = folder["name"]
        print(f"[{folder_name}]")

    for job in jobs:
        job_name = job["name"]
        print(f"{job_name}")



from itertools import islice

from rich.console import Console
from rich.table import Table

from jenkinsctl.configs.session import Session
from jenkinsctl.jenkins.console_util import format_timestamp
from jenkinsctl.jenkins.job import get_job, get_builds_iter
from jenkinsctl.jenkins.utils import normalize_job_path


def list_handler(session: Session, job_name: str, number: int):
    job_name = normalize_job_path(job_name)
    job = get_job(session, job_name)

    console = Console()
    table = Table(show_header=True, header_style="bold cyan", box=None)

    table.add_column("NUMBER", width=6)
    table.add_column("IN PROGRESS", width=15)
    table.add_column("RESULT", width=10)
    table.add_column("BUILDING", width=8)
    table.add_column("BRANCH", width=15)
    table.add_column("REVISION", width=8)
    table.add_column("STARTED BY", width=15)
    table.add_column("BUILD TIME", width=20)

    for build in islice(get_builds_iter(session, job), number):
        number = build["number"]
        building = build["building"]
        in_progress = build["inProgress"]
        result = build["result"]
        timestamp = build["timestamp"]

        actions = build["actions"]
        causes = next((action["causes"] for action in actions if action.get("causes") is not None), None)
        user_id = next((cause["userId"] for cause in causes if cause.get("userId") is not None), None)

        branch = None
        revision = None
        # fixme: not working below code
        # last_build = next((action["lastBuildRevision"] for action in actions if action.get("lastBuildRevision") is not None))
        last_build = None
        if last_build:
            branch_prefix = "refs/remotes/origin/"
            branch = last_build["branch"][0]["name"]
            if branch.startswith(branch_prefix):
                branch = branch[len(branch_prefix):]
            revision = last_build["SHA1"][:5]


        table.add_row(
            str(number),
            str(in_progress),
            str(result),
            str(building),
            branch,
            revision,
            user_id,
            str(format_timestamp(timestamp)))

    console.print(table)



import json

from rich.console import Console
from datetime import datetime
from dateutil import tz


def get_console():
    return Console()


def format_timestamp(epoch_timestamp):
    # Convert epoch timestamp to a naive datetime object
    naive_dt = datetime.fromtimestamp(epoch_timestamp / 1000)

    # Get the local timezone
    local_tz = tz.tzlocal()

    # Convert naive datetime to local timezone
    local_dt = naive_dt.astimezone(local_tz)

    # Format the datetime object to a string
    return local_dt.strftime('%Y-%m-%d %H:%M')


def print_json(dict_obj):
    json_str = json.dumps(dict_obj)
    get_console().print_json(json_str)

def json_preety(dict_obj):
    return json.dumps(dict_obj, indent=2)